#!/usr/bin/env python
# -*- coding:utf-8 -*-

import argparse
import os
import re
import sys
import warnings

from splinter.browser import Browser
from extractors.jasmine import Extractor as JExtractor
from extractors.qunit import Extractor as QExtractor
from reporters.stdout import print_result

class TestSuiteNotDetectedError(Exception):
    pass

def run_specs(path, browser_driver='firefox'):
    print
    print 'Using %s as runner and %s as webdriver.' % (path, browser_driver)

    browser = Browser(browser_driver)
    browser.visit(path)

    try:
        Extractor = filter(lambda e: e.is_it_me(browser), [JExtractor, QExtractor])[0]
    except IndexError:
        raise TestSuiteNotDetectedError('test suite not detected.')

    extractor = Extractor(browser)
    extractor.wait_till_finished_and_then(print_result)

    browser.quit()

    return extractor.failures_number


def has_scheme(uri):
    return bool(re.match(r'^[^:]+://', uri))


def main(args=sys.argv):
    ''' Runs Jasmine specs via console. '''
    current_directory = os.getcwd()
    default_runner_path = 'file://%s' % os.path.join(current_directory, 'SpecRunner.html')

    parser = argparse.ArgumentParser(description=u'Run your jasmine specs from command line using splinter')
    parser.add_argument('uri', metavar='URI', nargs='?', help='file path or url to runner file', default=None)
    parser.add_argument('-b', '--browser-driver', metavar='browser_driver', help='splinter driver to use', default='firefox')
    #deprecated args
    parser.add_argument('-f', '--filepath', metavar='FILEPATH', help='path to runner file (deprecated)', default=None)
    parser.add_argument('-u', '--url', metavar='URL', help='url to runner (deprecated)', default=None)

    args = args[1:]
    args = parser.parse_args(args)

    if args.uri:
        if has_scheme(args.uri):
            runner_path = args.uri
        else:
            runner_path = 'file://%s' % args.uri

    # deprecated options
    # when the deprecated options are removed this code can be removed
    elif args.url:
        runner_path = args.url
        warnings.warn(
            '-u and --url options are deprecated, use the url directly instead. Ex: `jasmine-splinter URL`',
            DeprecationWarning
        )
    elif args.filepath:
        runner_path = 'file://%s' % args.filepath
        warnings.warn(
            '-f and --filepath options are deprecated, use the filepath directly instead. Ex: `jasmine-splinter FILEPATH`',
            DeprecationWarning
        )
    #end of depreated options code

    else:
        runner_path = default_runner_path

    sys.exit(run_specs(runner_path, args.browser_driver))




#!/usr/bin/env python
# -*- coding:utf-8 -*-

from termcolor import colored


def print_result(extractor):
    if extractor.failures_number:
        print_errors(extractor)
    else:
        print_success(extractor)


def print_success(extractor):
    print
    print colored('✓ %s' % extractor.description, 'green')
    print


def print_errors(extractor):
    print
    print colored(extractor.description, 'yellow')
    print

    print format_errors(extractor.get_failures())
    print


def format_errors(errors):
    ret = []
    for error in errors:
        ret.extend(_print(error, [], 0))
    return '\n'.join(ret)


def _print(obj, buffer, level):
    for title, desc in obj.iteritems():
        buffer.append((level * 2 * ' ') + title)
        if desc and isinstance(desc[0], basestring):
            for err in desc:
                buffer.append(colored('%s  ✗ %s' % ((level * 2 * ' '), err), 'red'))
        else:
            level += 1
            for suite in desc:
                _print(suite, buffer, level)

    return buffer




#!/usr/bin/env python
# -*- coding:utf-8 -*-

import re

from jasmine_runner.extractors import BaseExtractor, class_xpath_to_css


class Extractor(BaseExtractor):

    @staticmethod
    def is_it_me(browser):
        return browser.is_element_present_by_css('.jasmine_reporter')

    def has_finished(self):
        return not self.browser.is_text_present('Running...')

    def has_failed(self):
        runner_div = self.browser.find_by_css('.runner').first
        return not 'passed' in runner_div['class']

    @property
    def failures_number(self):
        if hasattr(self, '_failures'):
            return self._failures

        if self.has_failed():
            self._failures = int(re.search(r'(\d+)\s*failure', self.description).group(1))
        else:
            self._failures = 0

        return self._failures

    @property
    def description(self):
        if hasattr(self, '_description'):
            return self._description

        self._description = self.browser.find_by_css('.runner .description').first.text.encode('utf8')
        return self._description

    def get_failures(self):
        rootDescribes = self.browser.find_by_xpath('//*%s/*%s%s' % (
            class_xpath_to_css('jasmine_reporter'),
            class_xpath_to_css('suite'),
            class_xpath_to_css('failed'),
        ))

        specs = []

        def traverse(describes, specs):
            for describe in describes:
                desc = describe.find_by_css('.description')
                spec = {}
                children = spec[desc.first.text.encode('utf8')] = []
                specs.append(spec)

                if 'suite' in describe['class']:
                    traverse(
                        describe.find_by_xpath('*%s' % class_xpath_to_css('failed')),
                        children
                    )
                elif 'spec' in describe['class']:
                    children.extend(map(lambda el: el.text.encode('utf8'), describe.find_by_css('.resultMessage')))

        traverse(rootDescribes, specs)

        return specs




#!/usr/bin/env python
# -*- coding:utf-8 -*-

from jasmine_runner.extractors import BaseExtractor, class_xpath_to_css


class Extractor(BaseExtractor):

    @staticmethod
    def is_it_me(browser):
        return browser.is_element_present_by_id('qunit-header')

    def has_finished(self):
        return self.browser.is_element_present_by_id('qunit-testresult')

    def has_failed(self):
        banner = self.browser.find_by_id('qunit-banner')
        return 'qunit-fail' in banner['class']

    @property
    def failures_number(self):
        if hasattr(self, '_failures'):
            return self._failures

        self._failures = int(self.browser.find_by_css('#qunit-testresult .failed').text)
        return self._failures

    @property
    def description(self):
        if hasattr(self, '_description'):
            return self._description

        self._description = self.browser.find_by_id('qunit-testresult').text
        return self._description

    def get_failures(self):

        specs = []

        fail_container = self.browser.find_by_id('qunit-tests')
        fail_suites = fail_container.find_by_xpath('*%s' % class_xpath_to_css('fail'))

        for suite in fail_suites:
            suite_description_node = suite.find_by_tag('strong').first
            # hack to make message "pickable"
            # some drivers (selenium) do not allow to get the text if the element is not visible
            suite_description_node.click()
            description = suite_description_node.text

            fail_messages = map(lambda el: el.text, suite.find_by_xpath('ol/li%s' % class_xpath_to_css('fail')))

            specs.append({
                description: fail_messages
            })

        return specs





from setuptools import setup, find_packages
from jasmine_runner import __version__

README = open('README.rst').read()

setup(name='jasmine-splinter-runner',
      version=__version__,
      description='jasmine runner based on splinter',
      long_description=README,
      author='CobraTeam',
      author_email='francisco@franciscosouza.net',
      packages=find_packages(),
      include_package_data=True,
      test_suite='nose.collector',
      install_requires=['argparse>=1.2.1,<1.3.0', 'splinter>=0.4.3,<0.5.0', 'termcolor>=1.1.0,<1.2.0'],
      entry_points = {
          'console_scripts' : [
              'jasmine-splinter = jasmine_runner.commands:main',
          ]
      },
     )




#!/usr/bin/env python
# -*- coding:utf-8 -*-

def class_xpath_to_css(_class):
    return '[contains(concat(" ",normalize-space(@class)," ")," %s ")]' % _class


class BaseExtractor(object):

    def __init__(self, browser):
        self.browser = browser

    def wait_till_finished_and_then(self, function):
        while not self.has_finished():
            pass
        function(self)

    def get_failures(self):
        '''
            this function returns an array with the following structure:
            [{'title of test suite':
                [{'title of nested test suite':
                    [{'spec description': ['spec error message']}, ...]
                , ...]
            , ...]
        '''
        raise NotImplemented





import os
import sys
import mocker

from jasmine_runner.commands import run_specs
from StringIO import StringIO

TESTS_ROOT = os.path.abspath(os.path.dirname(__file__))
FIXTURES_ROOT = os.path.join(TESTS_ROOT, 'fixtures')

path_to_file = lambda filename: 'file://%s' % (os.path.join(FIXTURES_ROOT, filename))

class TestJasmineRunner(mocker.MockerTestCase):

    def setUp(self):
        self._buf = StringIO()
        self._stdout = sys.stdout
        sys.stdout = self._buf

    def tearDown(self):
        sys.stdout = self._stdout
        self.mocker.reset()

    def assert_printed(self, value):
        self._buf.seek(0)
        assert value in self._buf.read()

    def _mock_colored_output(self, color):
        colored = self.mocker.replace('termcolor.colored')
        colored(mocker.ANY, color)
        self.mocker.result('bla')
        self.mocker.replay()

    def _mock_exit(self, w=mocker.ANY):
        exit = self.mocker.replace('sys.exit')
        exit(w)

    def test_should_print_the_resume_of_the_spec_running_for_passed_specs(self):
        "should print the resume of the spec running for passed specs"
        run_specs(path_to_file('passed-specs.html'))
        self.assert_printed('4 specs, 0 failures in 0.031s')

    def test_should_print_the_resume_of_the_spec_running_for_failed_specs(self):
        "should print the resume of the spec running for failed specs"
        run_specs(path_to_file('failed-specs.html'))
        self.assert_printed('4 specs, 2 failures in 0.028s')

    def test_green_resume(self):
        "should print a green resume for passed specs"
        self._mock_colored_output('green')

        run_specs(path_to_file('passed-specs.html'))
        self.assert_printed('bla')

        self.mocker.verify()

    def test_red_resume(self):
        "should print a red resume for failed specs"
        self._mock_colored_output('red')

        run_specs(path_to_file('failed-specs.html'))
        self.assert_printed('bla')

        self.mocker.verify()

    def test_splinter_driver(self):
        "should be able to customize the splinter driver to use"
        from splinter.browser import Browser
        chrome_mock = Browser('firefox')
        firefox_mock = Browser('firefox')

        Browser = self.mocker.replace('splinter.browser.Browser')
        Browser('chrome')
        self.mocker.result(chrome_mock)

        Browser('firefox')
        self.mocker.result(firefox_mock)
        self.mocker.replay()

        run_specs(path_to_file('failed-specs.html'), browser_driver='chrome')
        run_specs(path_to_file('passed-specs.html'), browser_driver='firefox')

        self.mocker.verify()

    def test_firefox_default_driver(self):
        "when no driver is specified, Firefox should be used"
        from splinter.browser import Browser
        browser = Browser('firefox')

        Browser = self.mocker.replace('splinter.browser.Browser')
        Browser('firefox')
        self.mocker.result(browser)
        self.mocker.replay()

        run_specs(path_to_file('passed-specs.html'))

        self.mocker.verify()

    def test_exit_status(self):
        "should return the proper exit status (very useful for continuous integration jobs)"
        assert 0 == run_specs(path_to_file('passed-specs.html'))
        assert 2 == run_specs(path_to_file('failed-specs.html'))

    def test_main_with_default_options(self):
        "should look for a SpecRunner.html file in the current directory by default"
        getcwd = self.mocker.replace('os.getcwd')
        getcwd()
        self.mocker.result(FIXTURES_ROOT)
        self._mock_exit()
        self.mocker.replay()

        from jasmine_runner.commands import main
        main(args=['jasmine-splinter'])

        self.mocker.verify()

    def test_specify_filepath(self):
        "should be able to specify the filepath"
        self._mock_exit(w=0)
        self.mocker.replay()

        from jasmine_runner.commands import main
        main(args=['jasmine-splinter', '--filepath=%s' % os.path.join(FIXTURES_ROOT, 'passed-specs.html')])

        self.mocker.verify()

    def test_specify_url(self):
        "should be able to specify the url of the runner"
        self._mock_exit(w=0)
        self.mocker.replay()

        from jasmine_runner.commands import main
        main(args=['jasmine-splinter', '--url=%s' % path_to_file('passed-specs.html')])

        self.mocker.verify()

    def test_choose_the_splinter_driver(self):
        "should be able to choose the splinter driver from command line"
        from splinter.browser import Browser
        browser = Browser('firefox')

        self._mock_exit(w=0)
        Browser = self.mocker.replace('splinter.browser.Browser')
        Browser('chrome')
        self.mocker.result(browser)
        self.mocker.replay()

        from jasmine_runner.commands import main
        main(args=['jasmine-splinter', '--browser-driver=chrome', '--url=%s' % path_to_file('passed-specs.html')])

        self.mocker.verify()

    def test_the_first_argument_can_be_a_file_path(self):
        "should be able to specify a url or file path"
        self._mock_exit(w=0)
        self.mocker.replay()

        from jasmine_runner.commands import main
        main(args=['jasmine-splinter', os.path.join(FIXTURES_ROOT, 'passed-specs.html')])

        self.mocker.verify()

    def test_the_first_argument_can_be_a_url(self):
        "should be able to specify a url or file path"
        self._mock_exit(w=0)
        self.mocker.replay()

        from jasmine_runner.commands import main
        main(args=['jasmine-splinter', path_to_file('passed-specs.html')])

        self.mocker.verify()


